@@ -32,6 +32,8 @@
 #include <libgda/gda-custom-marshal.h>
 #include <libgda/gda-types.h>
 
+#define __gda_value_is_null(value) (!G_IS_VALUE (value))
+
 /* 
  * Main static functions 
  */
@@ -132,7 +134,8 @@ gda_holder_get_type (void)
 			NULL,
 			sizeof (GdaHolder),
 			0,
-			(GInstanceInitFunc) gda_holder_init
+			(GInstanceInitFunc) gda_holder_init,
+			0
 		};
 		
 		g_static_mutex_lock (&registering);
@@ -145,10 +148,10 @@ gda_holder_get_type (void)
 }
 
 static gboolean
-validate_change_accumulator (GSignalInvocationHint *ihint,
+validate_change_accumulator (G_GNUC_UNUSED GSignalInvocationHint *ihint,
 			   GValue *return_accu,
 			   const GValue *handler_return,
-			   gpointer data)
+			   G_GNUC_UNUSED gpointer data)
 {
 	GError *error;
 
@@ -159,13 +162,14 @@ validate_change_accumulator (GSignalInvocationHint *ihint,
 }
 
 static GError *
-m_validate_change (GdaHolder *holder, const GValue *new_value)
+m_validate_change (G_GNUC_UNUSED GdaHolder *holder, G_GNUC_UNUSED const GValue *new_value)
 {
 	return NULL;
 }
 
 static void
-holder_attribute_set_cb (GObject *obj, const gchar *att_name, const GValue *value, gpointer data)
+holder_attribute_set_cb (GObject *obj, const gchar *att_name, const GValue *value,
+			 G_GNUC_UNUSED gpointer data)
 {
 	g_signal_emit (obj, gda_holder_signals[ATT_CHANGED], 0, att_name, value);
 }
@@ -339,7 +343,7 @@ gda_holder_new (GType type)
  * Note1: if @orig is set with a static value (see gda_holder_take_static_value()) 
  * its copy will have a fresh new allocated GValue, so that user should free it when done.
  *
- * Returns: a new #GdaHolder object
+ * Returns: (transfer full): a new #GdaHolder object
  */
 GdaHolder *
 gda_holder_copy (GdaHolder *orig)
@@ -585,11 +589,11 @@ gda_holder_set_property (GObject *object,
 				
 				/* updating the holder's validity regarding the NULL value */
 				if (!not_null && 
-				    (!holder->priv->value || gda_value_is_null (holder->priv->value)))
+				    (!holder->priv->value || __gda_value_is_null (holder->priv->value)))
 					holder->priv->valid = TRUE;
 				
 				if (not_null && 
-				    (!holder->priv->value || gda_value_is_null (holder->priv->value)))
+				    (!holder->priv->value || __gda_value_is_null (holder->priv->value)))
 					holder->priv->valid = FALSE;
 				
 				g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
@@ -612,6 +616,9 @@ gda_holder_set_property (GObject *object,
 		case PROP_SOURCE_COLUMN:
 			holder->priv->source_col = g_value_get_int (value);
 			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -663,6 +670,9 @@ gda_holder_get_property (GObject *object,
 		case PROP_SOURCE_COLUMN:
 			g_value_set_int (value, holder->priv->source_col);
 			break;	
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -673,7 +683,7 @@ gda_holder_get_property (GObject *object,
  * 
  * Get @holder's type
  *
- * Returns: (tranfer none): the data type
+ * Returns: (transfer none): the data type
  */
 GType
 gda_holder_get_g_type (GdaHolder *holder)
@@ -711,6 +721,8 @@ gda_holder_get_id (GdaHolder *holder)
  *
  * If @holder is set to NULL, then the returned value is a #GDA_TYPE_NULL GValue.
  *
+ * If @holder is invalid, then the returned value is %NULL.
+ *
  * Returns: the value, or %NULL
  */
 const GValue *
@@ -719,22 +731,26 @@ gda_holder_get_value (GdaHolder *holder)
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), NULL);
 	g_return_val_if_fail (holder->priv, NULL);
 
-	if (!holder->priv->full_bind) {
-		/* return default value if possible */
-		if (holder->priv->default_forced) {
-			g_assert (holder->priv->default_value);
-			if (G_VALUE_TYPE (holder->priv->default_value) == holder->priv->g_type) 
-				return holder->priv->default_value;
-			else
-				return NULL;
-		}
-
-		if (!holder->priv->value)
-			holder->priv->value = gda_value_new_null ();
-		return holder->priv->value;
-	}
-	else 
+	if (holder->priv->full_bind)
 		return gda_holder_get_value (holder->priv->full_bind);
+	else {
+		if (holder->priv->valid) {
+			/* return default value if possible */
+			if (holder->priv->default_forced) {
+				g_assert (holder->priv->default_value);
+				if (G_VALUE_TYPE (holder->priv->default_value) == holder->priv->g_type) 
+					return holder->priv->default_value;
+				else
+					return NULL;
+			}
+			
+			if (!holder->priv->value)
+				holder->priv->value = gda_value_new_null ();
+			return holder->priv->value;
+		}
+		else
+			return NULL;
+	}
 }
 
 /**
@@ -756,7 +772,7 @@ gda_holder_get_value_str (GdaHolder *holder, GdaDataHandler *dh)
 	g_return_val_if_fail (holder->priv, NULL);
 
 	current_val = gda_holder_get_value (holder);
-        if (!current_val || gda_value_is_null (current_val))
+        if (!current_val || __gda_value_is_null (current_val))
                 return NULL;
         else {
                 if (!dh)
@@ -908,11 +924,11 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 	}
 		
 	/* holder will be changed? */
-	newnull = !value || gda_value_is_null (value);
+	newnull = !value || __gda_value_is_null (value);
 	current_val = gda_holder_get_value (holder);
 	if (current_val == value)
 		changed = FALSE;
-	else if ((!current_val || gda_value_is_null ((GValue *)current_val)) && newnull)
+	else if ((!current_val || __gda_value_is_null ((GValue *)current_val)) && newnull)
 		changed = FALSE;
 	else if (value && current_val &&
 		 (G_VALUE_TYPE (value) == G_VALUE_TYPE ((GValue *)current_val)))
@@ -1042,11 +1058,11 @@ real_gda_holder_set_const_value (GdaHolder *holder, const GValue *value,
 #endif
 
 	/* holder will be changed? */
-	newnull = !value || gda_value_is_null (value);
+	newnull = !value || __gda_value_is_null (value);
 	current_val = gda_holder_get_value (holder);
 	if (current_val == value)
 		changed = FALSE;
-	else if ((!current_val || gda_value_is_null (current_val)) && newnull) 
+	else if ((!current_val || __gda_value_is_null (current_val)) && newnull) 
 		changed = FALSE;
 	else if (value && current_val &&
 		 (G_VALUE_TYPE (value) == G_VALUE_TYPE (current_val))) 
@@ -1220,7 +1236,6 @@ gda_holder_force_invalid (GdaHolder *holder)
 
 	holder->priv->invalid_forced = TRUE;
 	holder->priv->valid = FALSE;
-	
 	if (holder->priv->value) {
 		if (holder->priv->is_freeable)
 			gda_value_free (holder->priv->value);
@@ -1371,8 +1386,8 @@ gda_holder_set_default_value (GdaHolder *holder, const GValue *value)
 		const GValue *current = gda_holder_get_value (holder);
 
 		/* check if default is equal to current value */
-		if (gda_value_is_null (value) &&
-		    (!current || gda_value_is_null (current)))
+		if (__gda_value_is_null (value) &&
+		    (!current || __gda_value_is_null (current)))
 			holder->priv->default_forced = TRUE;
 		else if ((G_VALUE_TYPE (value) == holder->priv->g_type) &&
 			 current && !gda_value_compare (value, current))
@@ -1437,7 +1452,7 @@ gda_holder_get_not_null (GdaHolder *holder)
  */
 gboolean
 gda_holder_set_source_model (GdaHolder *holder, GdaDataModel *model,
-			     gint col, GError **error)
+			     gint col, G_GNUC_UNUSED GError **error)
 {
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 	g_return_val_if_fail (holder->priv, FALSE);
@@ -1506,13 +1521,15 @@ gda_holder_get_source_model (GdaHolder *holder, gint *col)
  * gda_holder_set_bind() was called, and it makes sure @holder's GType is the same as @holder->priv->simple_bind's
  */
 static void
-bind_to_notify_cb (GdaHolder *bind_to, GParamSpec *pspec, GdaHolder *holder)
+bind_to_notify_cb (GdaHolder *bind_to, G_GNUC_UNUSED GParamSpec *pspec, GdaHolder *holder)
 {
 	g_signal_handler_disconnect (holder->priv->simple_bind,
 				     holder->priv->simple_bind_notify_signal_id);
 	holder->priv->simple_bind_notify_signal_id = 0;
-	if (holder->priv->g_type == GDA_TYPE_NULL)
+	if (holder->priv->g_type == GDA_TYPE_NULL) {
 		holder->priv->g_type = bind_to->priv->g_type;
+		g_object_notify ((GObject*) holder, "g-type");
+	}
 	else if (holder->priv->g_type != bind_to->priv->g_type) {
 		/* break holder's binding because type differ */
 		g_warning (_("Cannot bind holders if their type is not the same, "
@@ -1633,12 +1650,12 @@ gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of)
 		g_return_if_fail (alias_of->priv);
 		g_return_if_fail (holder->priv->g_type == alias_of->priv->g_type);
 		cvalue = gda_holder_get_value (alias_of);
-		if (cvalue && !gda_value_is_null ((GValue*)cvalue))
+		if (cvalue && !__gda_value_is_null ((GValue*)cvalue))
 			value2 = gda_value_copy ((GValue*)cvalue);
 	}
 
 	cvalue = gda_holder_get_value (holder);
-	if (cvalue && !gda_value_is_null ((GValue*)cvalue))
+	if (cvalue && !__gda_value_is_null ((GValue*)cvalue))
 		value1 = gda_value_copy ((GValue*)cvalue);
 		
 	
