@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 2007 - 2008 Vivien Malerba
+/*
+ * Copyright (C) 2007 - 2009 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -36,7 +36,7 @@
 /*
  * Error reporting
  */
-GQuark 
+GQuark
 gda_sql_error_quark (void)
 {
         static GQuark quark;
@@ -45,11 +45,11 @@ gda_sql_error_quark (void)
         return quark;
 }
 
-/* 
- * statement's infos retreival
+/*
+ * statement's infos retrieval
  */
 GdaSqlStatementContentsInfo *
-gda_sql_statement_get_contents_infos  (GdaSqlStatementType type) 
+gda_sql_statement_get_contents_infos  (GdaSqlStatementType type)
 {
 	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 	static GdaSqlStatementContentsInfo **contents = NULL;
@@ -58,47 +58,70 @@ gda_sql_statement_get_contents_infos  (GdaSqlStatementType type)
 	if (!contents) {
 		contents = g_new0 (GdaSqlStatementContentsInfo *, GDA_SQL_STATEMENT_NONE);
 
-		contents [GDA_SQL_STATEMENT_SELECT] = gda_sql_statement_select_get_infos ();
-		contents [GDA_SQL_STATEMENT_INSERT] = gda_sql_statement_insert_get_infos ();
-		contents [GDA_SQL_STATEMENT_DELETE] = gda_sql_statement_delete_get_infos ();
-		contents [GDA_SQL_STATEMENT_UPDATE] = gda_sql_statement_update_get_infos ();
-		contents [GDA_SQL_STATEMENT_BEGIN] = gda_sql_statement_begin_get_infos ();
-		contents [GDA_SQL_STATEMENT_COMPOUND] = gda_sql_statement_compound_get_infos ();
-		contents [GDA_SQL_STATEMENT_COMMIT] = gda_sql_statement_commit_get_infos ();
-		contents [GDA_SQL_STATEMENT_ROLLBACK] = gda_sql_statement_rollback_get_infos ();
-		contents [GDA_SQL_STATEMENT_UNKNOWN] = gda_sql_statement_unknown_get_infos ();
-		contents [GDA_SQL_STATEMENT_SAVEPOINT] = gda_sql_statement_savepoint_get_infos ();
-		contents [GDA_SQL_STATEMENT_ROLLBACK_SAVEPOINT] = gda_sql_statement_rollback_savepoint_get_infos ();
-		contents [GDA_SQL_STATEMENT_DELETE_SAVEPOINT] = gda_sql_statement_delete_savepoint_get_infos ();
+		contents [GDA_SQL_STATEMENT_SELECT] = _gda_sql_statement_select_get_infos ();
+		contents [GDA_SQL_STATEMENT_INSERT] = _gda_sql_statement_insert_get_infos ();
+		contents [GDA_SQL_STATEMENT_DELETE] = _gda_sql_statement_delete_get_infos ();
+		contents [GDA_SQL_STATEMENT_UPDATE] = _gda_sql_statement_update_get_infos ();
+		contents [GDA_SQL_STATEMENT_BEGIN] = _gda_sql_statement_begin_get_infos ();
+		contents [GDA_SQL_STATEMENT_COMPOUND] = _gda_sql_statement_compound_get_infos ();
+		contents [GDA_SQL_STATEMENT_COMMIT] = _gda_sql_statement_commit_get_infos ();
+		contents [GDA_SQL_STATEMENT_ROLLBACK] = _gda_sql_statement_rollback_get_infos ();
+		contents [GDA_SQL_STATEMENT_UNKNOWN] = _gda_sql_statement_unknown_get_infos ();
+		contents [GDA_SQL_STATEMENT_SAVEPOINT] = _gda_sql_statement_savepoint_get_infos ();
+		contents [GDA_SQL_STATEMENT_ROLLBACK_SAVEPOINT] = _gda_sql_statement_rollback_savepoint_get_infos ();
+		contents [GDA_SQL_STATEMENT_DELETE_SAVEPOINT] = _gda_sql_statement_delete_savepoint_get_infos ();
 	}
 	g_static_mutex_unlock (&mutex);
 
 	return contents[type];
 }
 
-/*
+GType
+gda_sql_statement_get_type (void)
+{
+	static GType our_type = 0;
+
+	if (our_type == 0)
+		our_type = g_boxed_type_register_static ("GdaSqlStatement",
+			(GBoxedCopyFunc) gda_sql_statement_copy,
+			(GBoxedFreeFunc) gda_sql_statement_free);
+	return our_type;
+}
+
+/**
+ * gda_sql_statement_new
+ * @type: type of statement to create
  *
- * Statement
+ * Use this function to create a #GdaSqlStatement of the specified @type type.
  *
+ * Returns: a new #GdaSqlStatement
  */
 GdaSqlStatement *
 gda_sql_statement_new (GdaSqlStatementType type)
 {
 	GdaSqlStatement *stmt;
 	GdaSqlStatementContentsInfo *infos = gda_sql_statement_get_contents_infos (type);
-	
+
 	stmt = g_new0 (GdaSqlStatement, 1);
 	stmt->stmt_type = type;
 	if (infos && infos->construct) {
 		stmt->contents = infos->construct ();
 		GDA_SQL_ANY_PART (stmt->contents)->type = type;
 	}
-	else 
+	else
 		TO_IMPLEMENT;
 
 	return stmt;
 }
 
+/**
+ * gda_sql_statement_copy
+ * @stmt: a #GdaSqlStatement pointer
+ *
+ * Creates a copy of @stmt.
+ *
+ * Returns: a new #GdaSqlStatement
+ */
 GdaSqlStatement *
 gda_sql_statement_copy (GdaSqlStatement *stmt)
 {
@@ -132,18 +155,24 @@ gda_sql_statement_copy (GdaSqlStatement *stmt)
 	return copy;
 }
 
-void 
+/**
+ * gda_sql_statement_free
+ * @stmt: a #GdaSqlStatement pointer
+ *
+ * Releases any memory associated to @stmt.
+ */
+void
 gda_sql_statement_free (GdaSqlStatement *stmt)
 {
 	GdaSqlStatementContentsInfo *infos;
 
 	infos = gda_sql_statement_get_contents_infos (stmt->stmt_type);
 	g_free (stmt->sql);
-	
+
 	if (stmt->contents) {
 		if (infos && infos->free)
 			infos->free (stmt->contents);
-		else 
+		else
 			TO_IMPLEMENT;
 	}
 	if (stmt->validity_meta_struct)
@@ -152,6 +181,14 @@ gda_sql_statement_free (GdaSqlStatement *stmt)
 	g_free (stmt);
 }
 
+/**
+ * gda_sql_statement_type_to_string
+ * @type: a #GdaSqlStatementType value
+ *
+ * Converts a #GdaSqlStatementType to a string, see also gda_sql_statement_string_to_type()
+ *
+ * Returns: a constant string
+ */
 const gchar *
 gda_sql_statement_type_to_string (GdaSqlStatementType type)
 {
@@ -165,11 +202,19 @@ gda_sql_statement_type_to_string (GdaSqlStatementType type)
 		return infos->name;
 }
 
+/**
+ * gda_sql_statement_string_to_type
+ * @type: a string representing a #GdaSqlStatementType type
+ *
+ * Converts a string to a #GdaSqlStatementType value, see also gda_sql_statement_type_to_string()
+ *
+ * Returns: a #GdaSqlStatementType value
+ */
 GdaSqlStatementType
 gda_sql_statement_string_to_type (const gchar *type)
 {
 	g_return_val_if_fail (type, GDA_SQL_STATEMENT_NONE);
-	
+
 	switch (*type) {
 	case 'B':
 		return GDA_SQL_STATEMENT_BEGIN;
@@ -177,7 +222,7 @@ gda_sql_statement_string_to_type (const gchar *type)
 		if ((type[1] == 'O') && (type[2] == 'M') && (type[2] == 'P'))
 			return GDA_SQL_STATEMENT_COMPOUND;
 		else
-			return GDA_SQL_STATEMENT_COMMIT;	
+			return GDA_SQL_STATEMENT_COMMIT;
 	case 'D':
 		if (!strcmp (type, "DELETE"))
 			return GDA_SQL_STATEMENT_DELETE;
@@ -207,6 +252,14 @@ gda_sql_statement_string_to_type (const gchar *type)
 }
 
 
+/**
+ * gda_sql_statement_serialize
+ * @stmt: a #GdaSqlStatement pointer
+ *
+ * Creates a string representation of @stmt.
+ *
+ * Returns: a new string
+ */
 gchar *
 gda_sql_statement_serialize (GdaSqlStatement *stmt)
 {
@@ -229,14 +282,14 @@ gda_sql_statement_serialize (GdaSqlStatement *stmt)
 		g_string_append (string, str);
 		g_free (str);
 	}
-	else 
+	else
 		TO_IMPLEMENT;
 
 	g_string_append_c (string, '}');
 	str = string->str;
 	g_string_free (string, FALSE);
 	return str;
-	
+
 }
 
 
@@ -249,7 +302,6 @@ static gboolean foreach_check_validity (GdaSqlAnyPart *node, GdaSqlStatementChec
 static gboolean gda_sql_expr_check_validity (GdaSqlExpr *expr, GdaSqlStatementCheckValidityData *data, GError **error);
 static gboolean gda_sql_field_check_validity (GdaSqlField *field, GdaSqlStatementCheckValidityData *data, GError **error);
 static gboolean gda_sql_table_check_validity (GdaSqlTable *table, GdaSqlStatementCheckValidityData *data, GError **error);
-static gboolean gda_sql_function_check_validity (GdaSqlFunction *function, GdaSqlStatementCheckValidityData *data, GError **error);
 static gboolean gda_sql_select_field_check_validity (GdaSqlSelectField *field, GdaSqlStatementCheckValidityData *data, GError **error);
 static gboolean gda_sql_select_target_check_validity (GdaSqlSelectTarget *target, GdaSqlStatementCheckValidityData *data, GError **error);
 
@@ -292,10 +344,64 @@ gda_sql_statement_check_validity (GdaSqlStatement *stmt, GdaConnection *cnc, GEr
 		data.cnc = cnc;
 		data.store = gda_connection_get_meta_store (cnc);
 		data.mstruct = gda_meta_struct_new (data.store, GDA_META_STRUCT_FEATURE_NONE);
-		
+
 		/* attach the GdaMetaStruct to @stmt */
 		stmt->validity_meta_struct = data.mstruct;
-		retval = gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents), 
+		retval = gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents),
+						   (GdaSqlForeachFunc) foreach_check_validity, &data, error);
+		return retval;
+	}
+	else
+		return TRUE;
+}
+
+/**
+ * gda_sql_statement_check_validity_m
+ * @stmt: a #GdaSqlStatement pointer
+ * @mstruct: a #GdaMetaStruct object, or %NULL
+ * @error: a place to store errors, or %NULL
+ *
+ * If @mstruct is not %NULL, then checks that all the database objects referenced in the statement i
+ * actually referenced in @mstruct
+ *  (for example the table being updated in a UPDATE statement must exist in the
+ * connection's database for the check to succeed).
+ * This method sets the @stmt-&gt;validity_meta_struct attribute to @mstruct.
+ *
+ * If @mstruct is %NULL, then remove any information from a previous call to this method stored in @stmt. In this case,
+ * the @stmt-&gt;validity_meta_struct attribute is cleared.
+ *
+ * Also note that some parts of @stmt may be modified: for example leading and trailing spaces in aliases or
+ * objects names will be removed.
+ *
+ * Returns: TRUE if no error occurred
+ *
+ * Since: 4.2
+ */
+gboolean
+gda_sql_statement_check_validity_m (GdaSqlStatement *stmt, GdaMetaStruct *mstruct, GError **error)
+{
+	g_return_val_if_fail (stmt, FALSE);
+	g_return_val_if_fail (!mstruct || GDA_IS_META_STRUCT (mstruct), FALSE);
+
+	/* check the structure first */
+	if (!gda_sql_statement_check_structure (stmt, error))
+		return FALSE;
+
+	/* clear any previous setting */
+	gda_sql_statement_check_clean (stmt);
+
+	if (mstruct) {
+		GdaSqlStatementCheckValidityData data;
+		gboolean retval;
+
+		/* prepare data */
+		data.cnc = NULL;
+		data.store = NULL;
+		data.mstruct = g_object_ref (mstruct);
+
+		/* attach the GdaMetaStruct to @stmt */
+		stmt->validity_meta_struct = data.mstruct;
+		retval = gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents),
 						   (GdaSqlForeachFunc) foreach_check_validity, &data, error);
 		return retval;
 	}
@@ -369,7 +475,7 @@ foreach_check_validity (GdaSqlAnyPart *node, GdaSqlStatementCheckValidityData *d
 				function->function_name = NULL;
 			}
 		}
-		return gda_sql_function_check_validity (function, data, error);
+		return TRUE;
 	}
 	case GDA_SQL_ANY_SQL_SELECT_FIELD: {
 		GdaSqlSelectField *field = (GdaSqlSelectField*) node;
@@ -432,23 +538,23 @@ foreach_check_validity (GdaSqlAnyPart *node, GdaSqlStatementCheckValidityData *d
 }
 
 static gboolean
-gda_sql_expr_check_validity (GdaSqlExpr *expr, GdaSqlStatementCheckValidityData *data, GError **error)
+gda_sql_expr_check_validity (GdaSqlExpr *expr, G_GNUC_UNUSED GdaSqlStatementCheckValidityData *data, G_GNUC_UNUSED GError **error)
 {
 	if (!expr) return TRUE;
 	if (!expr->param_spec) return TRUE;
-	gda_sql_expr_check_clean (expr);
+	_gda_sql_expr_check_clean (expr);
 
 	/* 2 checks to do here:
 	 *  - try to find the data type from expr->param_spec->type using data->mstruct (not yet possible)
 	 *  - if expr->param_spec->type is NULL, then try to identify it (and expr->param_spec->g_type)
 	 *    using @expr context, set expr->param_spec->validity_meta_dict.
 	 */
-        TO_IMPLEMENT;
+        /*TO_IMPLEMENT;*/
         return TRUE;
 }
 
 void
-gda_sql_expr_check_clean (GdaSqlExpr *expr)
+_gda_sql_expr_check_clean (GdaSqlExpr *expr)
 {
 	if (!expr) return;
 	if (expr->param_spec && expr->param_spec->validity_meta_dict)
@@ -462,24 +568,22 @@ gda_sql_field_check_validity (GdaSqlField *field, GdaSqlStatementCheckValidityDa
 	GdaSqlTable *stable;
 
 	if (!field) return TRUE;
-	gda_sql_field_check_clean (field);
+	_gda_sql_field_check_clean (field);
 
-	if (!data->cnc) return TRUE;
-
-	for (any = GDA_SQL_ANY_PART(field)->parent; 
-	     any && (any->type != GDA_SQL_ANY_STMT_INSERT) && (any->type != GDA_SQL_ANY_STMT_UPDATE); 
+	for (any = GDA_SQL_ANY_PART(field)->parent;
+	     any && (any->type != GDA_SQL_ANY_STMT_INSERT) && (any->type != GDA_SQL_ANY_STMT_UPDATE);
 	     any = any->parent);
 	if (!any) {
 		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 			      "%s", _("GdaSqlField is not part of an INSERT or UPDATE statement"));
 		return FALSE;
 	}
-	
-	if (any->type == GDA_SQL_ANY_STMT_INSERT) 
+
+	if (any->type == GDA_SQL_ANY_STMT_INSERT)
 		stable = ((GdaSqlStatementInsert*) any)->table;
-	else if (any->type == GDA_SQL_ANY_STMT_UPDATE) 
+	else if (any->type == GDA_SQL_ANY_STMT_UPDATE)
 		stable = ((GdaSqlStatementUpdate*) any)->table;
-	else 
+	else
 		g_assert_not_reached ();
 
 	if (!stable) {
@@ -497,8 +601,8 @@ gda_sql_field_check_validity (GdaSqlField *field, GdaSqlStatementCheckValidityDa
 	GValue value;
 	memset (&value, 0, sizeof (GValue));
 	g_value_set_string (g_value_init (&value, G_TYPE_STRING), field->field_name);
-	tcol = gda_meta_struct_get_table_column (data->mstruct, 
-						 GDA_META_TABLE (stable->validity_meta_object), 
+	tcol = gda_meta_struct_get_table_column (data->mstruct,
+						 GDA_META_TABLE (stable->validity_meta_object),
 						 &value);
 	g_value_unset (&value);
 	field->validity_meta_table_column = tcol;
@@ -507,12 +611,12 @@ gda_sql_field_check_validity (GdaSqlField *field, GdaSqlStatementCheckValidityDa
 			     _("Column '%s' not found"), field->field_name);
 		return FALSE;
 	}
-	
+
         return TRUE;
 }
 
-void 
-gda_sql_field_check_clean (GdaSqlField *field)
+void
+_gda_sql_field_check_clean (GdaSqlField *field)
 {
 	if (!field) return;
 	field->validity_meta_table_column = NULL;
@@ -537,7 +641,7 @@ find_table_or_view (GdaSqlAnyPart *part, GdaSqlStatementCheckValidityData *data,
 		GdaSqlAnyPart *any;
 
 		for (any = part->parent; any && any->parent; any = any->parent);
-		if (!any) 
+		if (!any)
 			g_set_error (&lerror, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 				      "%s", _("GdaSqlSelectField is not part of a SELECT statement"));
 		else {
@@ -550,9 +654,9 @@ find_table_or_view (GdaSqlAnyPart *part, GdaSqlStatementCheckValidityData *data,
 						GdaSqlSelectTarget *target = (GdaSqlSelectTarget*) targets->data;
 						if (!target->as)
 							continue;
-						g_value_set_string (g_value_init (&value, G_TYPE_STRING), 
+						g_value_set_string (g_value_init (&value, G_TYPE_STRING),
 								    target->table_name);
-						dbo = gda_meta_struct_complement (data->mstruct, 
+						dbo = gda_meta_struct_complement (data->mstruct,
 										  GDA_META_DB_UNKNOWN,
 										  NULL, NULL, &value, NULL);
 						g_value_unset (&value);
@@ -594,7 +698,7 @@ gda_sql_table_check_validity (GdaSqlTable *table, GdaSqlStatementCheckValidityDa
 	GdaMetaDbObject *dbo;
 
 	if (!table) return TRUE;
-	gda_sql_table_check_clean (table);
+	_gda_sql_table_check_clean (table);
 
 	if (!table->table_name) {
 		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
@@ -614,29 +718,13 @@ gda_sql_table_check_validity (GdaSqlTable *table, GdaSqlStatementCheckValidityDa
 	return table->validity_meta_object ? TRUE : FALSE;
 }
 
-void 
-gda_sql_table_check_clean (GdaSqlTable *table)
+void
+_gda_sql_table_check_clean (GdaSqlTable *table)
 {
 	if (!table) return;
 	table->validity_meta_object = NULL;
 }
 
-static gboolean
-gda_sql_function_check_validity (GdaSqlFunction *function, GdaSqlStatementCheckValidityData *data, GError **error)
-{
-	if (!function) return TRUE;
-
-        TO_IMPLEMENT;
-        return TRUE;
-}
-
-void
-gda_sql_function_check_clean (GdaSqlFunction *function)
-{
-	if (!function) return;
-	function->validity_meta_function = NULL;
-}
-
 static gboolean
 gda_sql_select_field_check_validity (GdaSqlSelectField *field, GdaSqlStatementCheckValidityData *data, GError **error)
 {
@@ -645,9 +733,9 @@ gda_sql_select_field_check_validity (GdaSqlSelectField *field, GdaSqlStatementCh
 	gboolean starred_field = FALSE;
 
 	if (!field) return TRUE;
-	gda_sql_select_field_check_clean (field);
+	_gda_sql_select_field_check_clean (field);
 
-	if (!field->field_name) 
+	if (!field->field_name)
 		/* field is not a table.field */
 		return TRUE;
 
@@ -661,7 +749,7 @@ gda_sql_select_field_check_validity (GdaSqlSelectField *field, GdaSqlStatementCh
 		GSList *targets;
 		GdaMetaTableColumn *tcol = NULL;
 
-		for (any = GDA_SQL_ANY_PART(field)->parent; 
+		for (any = GDA_SQL_ANY_PART(field)->parent;
 		     any && (any->type != GDA_SQL_ANY_STMT_SELECT);
 		     any = any->parent);
 		if (!any) {
@@ -670,37 +758,41 @@ gda_sql_select_field_check_validity (GdaSqlSelectField *field, GdaSqlStatementCh
 			return FALSE;
 		}
 
-		for (targets = ((GdaSqlStatementSelect *)any)->from->targets; targets; targets = targets->next) {
-			GdaSqlSelectTarget *target = (GdaSqlSelectTarget *) targets->data;
-			if (!target->validity_meta_object && 
-			    !gda_sql_select_target_check_validity (target, data, error))
-				return FALSE;
-
-			g_value_set_string (g_value_init (&value, G_TYPE_STRING), field->field_name);
-			tcol = gda_meta_struct_get_table_column (data->mstruct, 
-								 GDA_META_TABLE (target->validity_meta_object),
-								 &value);
-			g_value_unset (&value);
-			if (tcol) {
-				/* found a candidate */
-				if (dbo) {
-					g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
-						     _("Could not identify table for field '%s'"), field->field_name);
+		if (((GdaSqlStatementSelect *)any)->from) {
+			for (targets = ((GdaSqlStatementSelect *)any)->from->targets;
+			     targets;
+			     targets = targets->next) {
+				GdaSqlSelectTarget *target = (GdaSqlSelectTarget *) targets->data;
+				if (!target->validity_meta_object &&
+				    !gda_sql_select_target_check_validity (target, data, error))
 					return FALSE;
+				
+				g_value_set_string (g_value_init (&value, G_TYPE_STRING), field->field_name);
+				tcol = gda_meta_struct_get_table_column (data->mstruct,
+									 GDA_META_TABLE (target->validity_meta_object),
+									 &value);
+				g_value_unset (&value);
+				if (tcol) {
+					/* found a candidate */
+					if (dbo) {
+						g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+							     _("Could not identify table for field '%s'"), field->field_name);
+						return FALSE;
+					}
+					dbo = target->validity_meta_object;
 				}
-				dbo = target->validity_meta_object;
+			}
+			if (!dbo) {
+				targets = ((GdaSqlStatementSelect *)any)->from->targets;
+				if (starred_field && targets && !targets->next)
+					/* only one target => it's the one */
+					dbo = ((GdaSqlSelectTarget*) targets->data)->validity_meta_object;
 			}
 		}
 		if (!dbo) {
-			targets = ((GdaSqlStatementSelect *)any)->from->targets;
-			if (starred_field && targets && !targets->next) 
-				/* only one target => it's the one */
-				dbo = ((GdaSqlSelectTarget*) targets->data)->validity_meta_object;
-			else {
-				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
-					     _("Could not identify table for field '%s'"), field->field_name);
-				return FALSE;
-			}
+			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+				     _("Could not identify table for field '%s'"), field->field_name);
+			return FALSE;
 		}
 		field->validity_meta_object = dbo;
 		field->validity_meta_table_column = tcol;
@@ -717,13 +809,13 @@ gda_sql_select_field_check_validity (GdaSqlSelectField *field, GdaSqlStatementCh
 		field->validity_meta_object = dbo;
 		if (!field->validity_meta_object)
 			return FALSE;
-		
+
 		/* field part */
 		if (!gda_identifier_equal (field->field_name, "*")) {
 			GdaMetaTableColumn *tcol;
 			g_value_set_string (g_value_init (&value, G_TYPE_STRING), field->field_name);
-			tcol = gda_meta_struct_get_table_column (data->mstruct, 
-								 GDA_META_TABLE (field->validity_meta_object), 
+			tcol = gda_meta_struct_get_table_column (data->mstruct,
+								 GDA_META_TABLE (field->validity_meta_object),
 								 &value);
 			g_value_unset (&value);
 			field->validity_meta_table_column = tcol;
@@ -737,8 +829,8 @@ gda_sql_select_field_check_validity (GdaSqlSelectField *field, GdaSqlStatementCh
 	return TRUE;
 }
 
-void 
-gda_sql_select_field_check_clean (GdaSqlSelectField *field)
+void
+_gda_sql_select_field_check_clean (GdaSqlSelectField *field)
 {
 	if (!field) return;
 	field->validity_meta_object = NULL;
@@ -765,8 +857,8 @@ gda_sql_select_target_check_validity (GdaSqlSelectTarget *target, GdaSqlStatemen
 	return target->validity_meta_object ? TRUE : FALSE;
 }
 
-void 
-gda_sql_select_target_check_clean (GdaSqlSelectTarget *target)
+void
+_gda_sql_select_target_check_clean (GdaSqlSelectTarget *target)
 {
 	if (!target) return;
 	target->validity_meta_object = NULL;
@@ -786,7 +878,7 @@ gda_sql_statement_check_clean (GdaSqlStatement *stmt)
 	g_return_if_fail (stmt);
 
 	if (stmt->validity_meta_struct) {
-		gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents), 
+		gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents),
 					  (GdaSqlForeachFunc) foreach_check_clean, NULL, NULL);
 		g_object_unref (stmt->validity_meta_struct);
 		stmt->validity_meta_struct = NULL;
@@ -794,28 +886,25 @@ gda_sql_statement_check_clean (GdaSqlStatement *stmt)
 }
 
 static gboolean
-foreach_check_clean (GdaSqlAnyPart *node, gpointer data, GError **error)
+foreach_check_clean (GdaSqlAnyPart *node, G_GNUC_UNUSED gpointer data, G_GNUC_UNUSED GError **error)
 {
 	if (!node) return TRUE;
 
 	switch (node->type) {
 	case GDA_SQL_ANY_EXPR:
-		gda_sql_expr_check_clean ((GdaSqlExpr*) node);
+		_gda_sql_expr_check_clean ((GdaSqlExpr*) node);
 		break;
 	case GDA_SQL_ANY_SQL_FIELD:
-		gda_sql_field_check_clean ((GdaSqlField*) node);
+		_gda_sql_field_check_clean ((GdaSqlField*) node);
 		break;
 	case GDA_SQL_ANY_SQL_TABLE:
-		gda_sql_table_check_clean ((GdaSqlTable*) node);
-		break;
-	case GDA_SQL_ANY_SQL_FUNCTION:
-		gda_sql_function_check_clean ((GdaSqlFunction*) node);
+		_gda_sql_table_check_clean ((GdaSqlTable*) node);
 		break;
 	case GDA_SQL_ANY_SQL_SELECT_FIELD:
-		gda_sql_select_field_check_clean ((GdaSqlSelectField*) node);
+		_gda_sql_select_field_check_clean ((GdaSqlSelectField*) node);
 		break;
 	case GDA_SQL_ANY_SQL_SELECT_TARGET:
-		gda_sql_select_target_check_clean ((GdaSqlSelectTarget*) node);
+		_gda_sql_select_target_check_clean ((GdaSqlSelectTarget*) node);
 		break;
 	default:
 		break;
@@ -833,7 +922,7 @@ static gboolean foreach_check_struct (GdaSqlAnyPart *node, gpointer data, GError
  *
  * Checks for any error in @stmt's structure to make sure the statement is valid
  * (for example a SELECT statement must at least return a column, a DELETE statement must specify which table
- * is targetted).
+ * is targeted).
  *
  * Returns: TRUE if no error occurred
  */
@@ -842,16 +931,27 @@ gda_sql_statement_check_structure (GdaSqlStatement *stmt, GError **error)
 {
 	g_return_val_if_fail (stmt, FALSE);
 
-	return gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents), 
+	return gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents),
 					 (GdaSqlForeachFunc) foreach_check_struct, NULL, error);
 }
 
 static gboolean
-foreach_check_struct (GdaSqlAnyPart *node, gpointer data, GError **error)
+foreach_check_struct (GdaSqlAnyPart *node, G_GNUC_UNUSED gpointer data, GError **error)
 {
 	return gda_sql_any_part_check_structure (node, error);
 }
 
+/**
+ * gda_sql_any_part_check_structure
+ * @node: a #GdaSqlAnyPart pointer
+ * @error: a place to store errors, or %NULL
+ *
+ * Checks for any error in @node's structure to make sure it is valid. This
+ * is the same as gda_sql_statement_check_structure() but for individual #GdaSqlAnyPart
+ * parts. This function is mainly for database provider's implementations
+ *
+ * Returns: TRUE if no error occurred
+ */
 gboolean
 gda_sql_any_part_check_structure (GdaSqlAnyPart *node, GError **error)
 {
@@ -1072,10 +1172,10 @@ gda_sql_any_part_check_structure (GdaSqlAnyPart *node, GError **error)
 }
 
 /**
- * gda_sql_any_part_foreach
+ * gda_sql_any_part_foreach:
  * @node: the stat node
- * @func: function to call for each sub node
- * @data: data to pass to @func each time it is called
+ * @func: (scope call): function to call for each sub node
+ * @data: (closure): data to pass to @func each time it is called
  * @error: a place to store errors, or %NULL (is also passed to @func)
  *
  * Calls a function for each element of a #GdaSqlAnyPart node
@@ -1088,7 +1188,7 @@ gda_sql_any_part_foreach (GdaSqlAnyPart *node, GdaSqlForeachFunc func, gpointer
 {
 	GSList *l;
 	if (!node) return TRUE;
-	
+
 	/* call @func for the node's contents */
 	switch (node->type) {
 	case GDA_SQL_ANY_STMT_BEGIN:
@@ -1169,7 +1269,7 @@ gda_sql_any_part_foreach (GdaSqlAnyPart *node, GdaSqlForeachFunc func, gpointer
 	case GDA_SQL_ANY_STMT_COMPOUND: {
 		GdaSqlStatementCompound *stmt = (GdaSqlStatementCompound*) node;
 		for (l = stmt->stmt_list; l; l = l->next)
-			if (!gda_sql_any_part_foreach (GDA_SQL_ANY_PART (((GdaSqlStatement*) l->data)->contents), 
+			if (!gda_sql_any_part_foreach (GDA_SQL_ANY_PART (((GdaSqlStatement*) l->data)->contents),
 						       func, data, error))
 				return FALSE;
 		break;
@@ -1181,7 +1281,7 @@ gda_sql_any_part_foreach (GdaSqlAnyPart *node, GdaSqlForeachFunc func, gpointer
 				return FALSE;
 		break;
 	}
-	
+
 	/* individual parts */
 	case GDA_SQL_ANY_EXPR: {
 		GdaSqlExpr *expr = (GdaSqlExpr*) node;
@@ -1292,7 +1392,7 @@ gda_sql_statement_normalize (GdaSqlStatement *stmt, GdaConnection *cnc, GError *
 	if (!stmt->validity_meta_struct && !gda_sql_statement_check_validity (stmt, cnc, error))
 		return FALSE;
 
-	retval = gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents), 
+	retval = gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents),
 					   (GdaSqlForeachFunc) foreach_normalize, cnc, error);
 #ifdef GDA_DEBUG
 	GError *lerror = NULL;
@@ -1307,7 +1407,7 @@ gda_sql_statement_normalize (GdaSqlStatement *stmt, GdaConnection *cnc, GError *
 }
 
 static gboolean
-foreach_normalize (GdaSqlAnyPart *node, GdaConnection *cnc, GError **error)
+foreach_normalize (GdaSqlAnyPart *node, G_GNUC_UNUSED GdaConnection *cnc, GError **error)
 {
 	if (!node) return TRUE;
 
@@ -1340,7 +1440,7 @@ foreach_normalize (GdaSqlAnyPart *node, GdaConnection *cnc, GError **error)
 				nfield->expr = gda_sql_expr_new ((GdaSqlAnyPart*) nfield);
 				nfield->expr->value = gda_value_new (G_TYPE_STRING);
 				if (field->table_name)
-					g_value_take_string (nfield->expr->value, g_strdup_printf ("%s.%s", 
+					g_value_take_string (nfield->expr->value, g_strdup_printf ("%s.%s",
 												   nfield->table_name,
 												   nfield->field_name));
 				else
@@ -1352,8 +1452,8 @@ foreach_normalize (GdaSqlAnyPart *node, GdaConnection *cnc, GError **error)
 					GSList *lnode = g_slist_nth (expr_list, nodepos);
 					lnode->data = nfield;
 				}
-				else 
-					((GdaSqlStatementSelect*) parent_node)->expr_list = 
+				else
+					((GdaSqlStatementSelect*) parent_node)->expr_list =
 						g_slist_insert (expr_list, nfield, ++nodepos);
 			}
 			/* get rid of @field */
